@@ -5,8 +5,8 @@ Preliminary documentation at https://github.com/ipython/ipython/wiki/IPEP-16%3A-
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
 import json
-import logging
 from textwrap import dedent
+from traceback import format_tb
 
 from ipython_genutils.py3compat import cast_unicode
 from jupyter_client import protocol_version as client_protocol_version
@@ -79,7 +79,10 @@ class KernelActionHandler(APIHandler):
             try:
                 await km.restart_kernel(kernel_id)
             except Exception as e:
-                self.log.error("Exception restarting kernel", exc_info=True)
+                message = "Exception restarting kernel"
+                self.log.error(message, exc_info=True)
+                traceback = format_tb(e.__traceback__)
+                self.write(json.dumps(dict(message=message, traceback=traceback)))
                 self.set_status(500)
             else:
                 model = await ensure_async(km.kernel_model(kernel_id))
@@ -326,6 +329,15 @@ class ZMQChannelsHandler(AuthenticatedZMQStreamHandler):
         # We don't want to wait forever, because browsers don't take it well when
         # servers never respond to websocket connection requests.
         kernel = self.kernel_manager.get_kernel(self.kernel_id)
+
+        if hasattr(kernel, "ready"):
+            try:
+                await kernel.ready
+            except Exception as e:
+                kernel.execution_state = "dead"
+                kernel.reason = str(e)
+                raise web.HTTPError(500, str(e)) from e
+
         self.session.key = kernel.session.key
         future = self.request_kernel_info()
 
@@ -446,6 +458,7 @@ class ZMQChannelsHandler(AuthenticatedZMQStreamHandler):
     def _on_zmq_reply(self, stream, msg_list):
         idents, fed_msg_list = self.session.feed_identities(msg_list)
         msg = self.session.deserialize(fed_msg_list)
+
         parent = msg["parent_header"]
 
         def write_stderr(error_message):
@@ -624,11 +637,11 @@ class ZMQChannelsHandler(AuthenticatedZMQStreamHandler):
         self.write_message(json.dumps(msg, default=json_default))
 
     def on_kernel_restarted(self):
-        logging.warn("kernel %s restarted", self.kernel_id)
+        self.log.warning("kernel %s restarted", self.kernel_id)
         self._send_status_message("restarting")
 
     def on_restart_failed(self):
-        logging.error("kernel %s restarted failed!", self.kernel_id)
+        self.log.error("kernel %s restarted failed!", self.kernel_id)
         self._send_status_message("dead")
 
     def _on_error(self, msg):
